@@ -1,16 +1,13 @@
 # -*- perl -*-
 #
-#  Net::Server - Extensible Perl internet server
+#  Net::Server
+#    ABSTRACT: Extensible Perl internet server
 #
-#  $Id: Server.pm,v 1.158 2013/01/10 07:37:17 rhandom Exp $
+#  Copyright (C) 2001-2017
 #
-#  Copyright (C) 2001-2013
+#    Paul Seamons <paul@seamons.com>
 #
-#    Paul Seamons
-#    paul@seamons.com
-#    http://seamons.com/
-#
-#    Rob Brown bbb@cpan,org
+#    Rob Brown <bbb@cpan,org>
 #
 #  This package may be distributed under the terms of either the
 #  GNU General Public License
@@ -32,7 +29,7 @@ use Net::Server::Proto ();
 use Net::Server::Daemonize qw(check_pid_file create_pid_file safe_fork
                               get_uid get_gid set_uid set_gid);
 
-our $VERSION = '2.007';
+our $VERSION = '2.010';
 
 sub new {
     my $class = shift || die "Missing class";
@@ -89,7 +86,7 @@ sub _initialize {
     my $self = shift;
     my $prop = $self->{'server'} ||= {};
 
-    $self->commandline($self->_get_commandline) if ! eval { $self->commandline }; # save for a HUP
+    $self->commandline($self->_get_commandline) if ! eval { local $SIG{__DIE__}; $self->commandline }; # save for a HUP
     $self->configure_hook;      # user customizable hook
     $self->configure;           # allow for reading of commandline, program, and configuration file parameters
 
@@ -531,7 +528,6 @@ sub get_client_info {
             ($prop->{'peerport'}, $addr) = Socket::sockaddr_in($prop->{'udp_peer'});
             $prop->{'peeraddr'} = Socket::inet_ntoa($addr);
         } else {
-            warn "Right here\n";
             ($prop->{'peerport'}, $addr) = Socket6::sockaddr_in6($prop->{'udp_peer'});
             $prop->{'peeraddr'} = Socket6->can('inet_ntop')
                                 ? Socket6::inet_ntop($client->sockdomain, $addr)
@@ -1037,6 +1033,8 @@ sub _read_conf {
 
 sub other_child_died_hook {}
 
+sub delete_child_hook {}
+
 sub delete_child {
     my ($self, $pid) = @_;
     my $prop = $self->{'server'};
@@ -1050,6 +1048,8 @@ sub delete_child {
             $prop->{'children'}->{$pid}->{'sock'}->close;
         }
     }
+    
+    $self->delete_child_hook($pid);   # user customizable hook
 
     delete $prop->{'children'}->{$pid};
 }
